// ============================================================================================
// ===																						===
// === cannon.c																				===
// ===																						===
// === Functions defined:																	===
// ===		phreshold_key																	===
// ===		start_new_cannon																===
// ===		move_starting_cannon															===
// ===		draw_cannon																		===
// ===		cannon_is_hit																	===
// ===																						===
// ============================================================================================

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/kernel.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "phreshold.h"
#include "abimport.h"
#include "proto.h"

/*--- Common definitions shared by the two tasks: phreshold and phreshold_kbd ---*/
#include "phreshold_kbd.h"
extern	KEYS_SHARED		*keys_state;


/*--- -------------------------------------------------------------------- ---*/
/*--- Variables used only by start_new_cannon() and move_starting_cannon() ---*/
/*--- -------------------------------------------------------------------- ---*/

static	int					starting_cannon_where;
static	STARTING_CANNON		*starting_cannon_which;
static	double				starting_cannon_par;


void start_new_cannon( void )
	{

	/*--- Which cannon to start ? ---*/
	starting_cannon_which = &starting_cannons[ --nb_cannons_left ];
	starting_cannon_where = 0;
	starting_cannon_par   = 10;
	is_starting_cannon = TRUE;

	}	// start_new_cannon


void move_starting_cannon( void )
	{

	/*--- Set the palette used by ships, missile, shells, cannon, etc. ---*/
	PgSetPalette( palette, 0, 0, 256, Pg_PALSET_SOFT, 0 );
	PgFlush();

	/*--- Prior to start, blink the cannon several time ---*/
	if ( ++starting_cannon_where < 180 )
		{
		if ( ((int)(starting_cannon_where / starting_cannon_par) % 2) == 0 )
			PtUnrealizeWidget( starting_cannon_which->wgt );
		else
			{
			PtRealizeWidget( starting_cannon_which->wgt );
			starting_cannon_par -= 0.085;
			}
		PtFlush();
		PgSetPalette( palette, 0, 0, 256, Pg_PALSET_SOFT, 0 );
		PgFlush();
		return;
		}
	else
		if ( starting_cannon_where == 180 )
			{
			PtRealizeWidget( starting_cannon_which->wgt );
			starting_cannon_par = 1;
			}

	/*--- Move the starting cannon ---*/
	setPos( starting_cannon_which->wgt,
		starting_cannon_which->x, starting_cannon_which->y );
	PtFlush();
	if ( (starting_cannon_which->y -= starting_cannon_par) <= -35 )
		{
		is_starting_cannon = FALSE;
		PgSetPalette( palette, 0, 0,  256, Pg_PALSET_SOFT, 0 );
		PgFlush();
		return;
		}
	starting_cannon_par += 0.099;

	PgSetPalette( palette, 0, 0, 256, Pg_PALSET_SOFT, 0 );
	PgFlush();

	}	// move_starting_cannon


void draw_cannon( void )
	{
	int		sz;

	/*--- Move the cannon ? ---*/
	if ( keys_state->left )
		{
		if ( (cannon.x -= CANNON_XINC) < 0 )
			cannon.x = 0;
		}
	else if ( keys_state->right )
		{
		if ( (cannon.x += CANNON_XINC) > (frame_width - cannon.w) )
			cannon.x = (frame_width - cannon.w);
		}
	else if ( keys_state->shoot )
		{
		if ( !cannon.is_hit )
			if ( !missile.active && !laser.overheat )
				launch_missile();
		}
	else if ( keys_state->escape )
		{
		if ( keys_state->escape_down )
			if ( !is_frozen )
				freeze();
			else
				unfreeze();
		}
	else if ( keys_state->exit )
		{
		PtDestroyWidget( ABW_phreshold );
		exit( 0 );
		}

	/*--- Draw the cannon to the frame buffer ---*/
	if ( !cannon.is_hit )
		{

		if ( cannon.state != CANNON_UNARMED )
			{
			if ( !(cannon.cnt++ % 10) )
				{
				if ( cannon.inc == 1 )
					{
					if ( ++cannon.state == 3 )
						{
						cannon.state -= 2;
						cannon.inc = -1;
						}
					}
				else
					{
					if ( !cannon.state-- )
						{
						cannon.state += 2;
						cannon.inc = 1;
						}
					}	
				}	// if
			}
		blaster_or( cannon_img[ cannon.state ],
			cannon.w, cannon.h,
			cannon.x, cannon.y );

		}
	else
		{

		sz = (cannon.w * cannon.h);
		make_hit_bitmap( cannon_img[ CANNON_DEAD ],
			sz, sz - (sz * --cannon.hitcnt) / CANNON_DESTROYED_IN );

		blaster_or( hitbmp,
			cannon.w, cannon.h,
			cannon.x, cannon.y );

		}

	}	// draw_cannon


void cannon_is_hit( void )
	{

	if ( testing_level == -1 )
		{

		cannon.is_hit = TRUE;
		cannon.hitcnt = CANNON_DESTROYED_IN;

		is_left = is_right = FALSE;
		
		}	// if

	}	// cannon_is_hit
