// ============================================================================================
// ===																						===
// === menu.c																				===
// ===																						===
// === Functions defined:																	===
// ===		about_setup																		===
// ===		menu_about_phreshold															===
// ===		menu_file_exit																	===
// ===		restart_same_level																===
// ===		start_new_level																	===
// ===		start_play																		===
// ===		menu_file_play																	===
// ===																						===
// ============================================================================================

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "phreshold.h"
#include "abimport.h"
#include "proto.h"


int
about_setup( PtWidget_t *link_instance, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	link_instance = link_instance, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Show the version number and the date of the software ---*/
	setString( ABW_about_version, "Version %d.%d.%c (%04d/%02d/%02d)",
		(VERSION_NB >> 16), (VERSION_NB >> 8) & 0xFF, (VERSION_NB & 0xFF) + 'a',
		(VERSION_DATE >> 16) + 1998, (VERSION_DATE >> 8) & 0xFF, (VERSION_DATE & 0xFF) );

	return( Pt_CONTINUE );

	}	// about_setup


int
menu_about_phreshold( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Open the 'About' window, only of already not opened ---*/
	if ( ABW_about == NULL )
		ApCreateModule( ABM_about, NULL, NULL );
	else
		window_focus( ABW_about );

	return( Pt_CONTINUE );

	}	// menu_about_phreshold


int
menu_file_exit( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Return to the OS ---*/
	PtDestroyWidget( ABW_phreshold );
	exit( 0 );

	return( Pt_CONTINUE );

	}	// menu_file_exit


/*--- ----------------------------------- ---*/
/*--- Initializations prior to a new play ---*/
/*--- ----------------------------------- ---*/

void restart_same_level( const IBOOL start_a_cannon )
	{
	SHELL				*shell;
	int					n;

	/*--- Prepare the frame: get the background, w/o objects ---*/
	memset( background, 0, frame_size );

	/*--- Reset shells ---*/
	nb_active_shells = 0;
	for ( shell = shells, n = 0; n < MAXNB_SHELLS; n++, shell++ )
		shell->active = FALSE;

	if ( start_a_cannon )
		{
	
		/*--- Start a new cannon ---*/	
		if ( testing_level == -1 )
			start_new_cannon();

		/*--- Initialize the initial position/state of the cannon ---*/
		cannon.state = CANNON_ARMED_0;
		cannon.inc = 1;
		cannon.x = (frame_width - cannon.w) >> 1;
		cannon.y = (frame_height - cannon.h);
		cannon.is_hit = FALSE;
		
		/*--- Laser ---*/
		laser.level = 0;
		laser.overheat = FALSE;
		laser.prev_clock = clock();
		laser.per_clock = (double)LASER_DEC / (double)CLOCKS_PER_SEC;
	
		}	// if

	/*--- No missile running ---*/
	missile.active = FALSE;

	/*--- State of the keys (used for the cannon movement) ---*/
	is_left  = FALSE;
	is_right = FALSE;

	/*--- This part CONF_LEVEL is filled at run-time ---*/
	currlevel.nb_ships_attacking = 0;		// Nb of ships attacking now

	/*--- Set the palette used by ships, missile, shells, cannon, etc. ---*/
	PgSetPalette( palette, 0, 0, 256, Pg_PALSET_SOFT, crc_palette );
	PgFlush();

	}	// restart_same_level


static void mother_ship( void )
	{
	struct timespec	pause;
	int				dx;
	double			nb_px, new_cannon_x;
	double			yinc;
	double			y;
	int				nb;

	/*--- Where to put the mother ship ---*/
	mother.y = -mother.h;
	mother.x = (frame_width - mother.w)/2;

	/*--- ------------------------- ---*/
	/*--- Move down the mother ship ---*/
	/*--- ------------------------- ---*/

	y = mother.y;
	yinc = 3;
	nb = 0;
	while ( ((int)y + mother.h + cannon.h) < frame_height )
		{
		y += yinc;
		yinc -= 0.0035;
		nb++;
		}
	
	/*--- Compute how many pixels to move the cannon ---*/
	dx = (frame_width - cannon.w)/2 - cannon.x;
	nb_px = (double)dx / (double)nb;
	
	is_scrolling_bt = FALSE;		// Scrolling bottom-top
	pause.tv_sec  = 0;
	pause.tv_nsec = 1000000;		// 1 msec
	new_cannon_x = cannon.x;
	y = mother.y;
	yinc = 3;
	while ( (mother.y + mother.h + cannon.h) < frame_height )
		{

		/*--- Move the mother ---*/
		PgSetPalette( palette, 0, 0, 256, Pg_PALSET_SOFT, crc_palette );
		memmove( work_frame, background, frame_size );
		blaster_or( mother_img,  mother.w, mother.h,  mother.x, mother.y = (int)y );
		y += yinc;

		/*--- Move the cannon ---*/
		new_cannon_x += nb_px;
		blaster_or( cannon_img[ cannon.state ],
			cannon.w, cannon.h,
			(int)new_cannon_x, cannon.y );

		/*--- Draw frame + little tempo ---*/		
		PgSetRegion( PtWidgetRid( ABW_phreshold ) );
		PgDrawImagemx( work_frame, Pg_IMAGE_PALETTE_BYTE, &frame_pos, &frame_dim, frame_width, 0 );
		PgFlush();
		nanosleep( &pause, NULL );
		yinc -= 0.0035;

		}	// while


	/*--- ---------- ---*/
	/*--- Fuel Bonus ---*/
	/*--- ---------- ---*/

	while ( getGauge_value( ABW_phreshold_fuel ) > 0 )
		{
		setGauge_value( ABW_phreshold_fuel, 
			getGauge_value( ABW_phreshold_fuel ) - CLOCKS_PER_SEC );
		score += 5;
		draw_score();
		nanosleep( &pause, NULL );
		}


	/*--- ----------- ---*/
	/*--- Refill Fuel ---*/
	/*--- ----------- ---*/

	setGauge_MinMax( ABW_phreshold_fuel, 0, 1000, 0 );
	while ( getGauge_value( ABW_phreshold_fuel ) != 1000 )
		{
		setGauge_value( ABW_phreshold_fuel, 
			getGauge_value( ABW_phreshold_fuel ) + 1 );
		nanosleep( &pause, NULL );
		}


	/*--- ----------------------- ---*/
	/*--- Move up the mother ship ---*/
	/*--- ----------------------- ---*/
	
	cannon.x = (frame_width - cannon.w)/2;
	yinc = 1;
	y = mother.y;
	while ( mother.y > -mother.h )
		{

		/*--- Move the mother ---*/
		PgSetPalette( palette, 0, 0, 256, Pg_PALSET_SOFT, crc_palette );
		memmove( work_frame, background, frame_size );
		blaster_or( mother_img,  mother.w, mother.h,  mother.x, mother.y = (int)y);
		y -= yinc;

		/*--- Move the cannon ---*/
		blaster_or( cannon_img[ cannon.state ],
			cannon.w, cannon.h,
			cannon.x, cannon.y );

		/*--- Draw frame + little tempo ---*/		
		PgSetRegion( PtWidgetRid( ABW_phreshold ) );
		PgDrawImagemx( work_frame, Pg_IMAGE_PALETTE_BYTE, &frame_pos, &frame_dim, frame_width, 0 );
		PgFlush();
		nanosleep( &pause, NULL );
		yinc += 0.0125;

		}	// while

	}	// mother_ship


void start_new_level( const IBOOL start_a_cannon )
	{

	/*--- Stop drawing frame ---*/
	refresh_frame( FALSE );

	/*--- Mother ship ? ---*/
	if ( testing_level == -1 )
		if ( (++level != 0) && ((level % 4) == 0) )
			mother_ship();

#if LOGTOTMP==1
	slog( "start playing level #%d", level+1 );
#endif

	/*--- Finish, no more level available ? ---*/
	if ( level == nb_levels )
		{
		setString( ABW_phreshold_msg, 
			"Sorry, no more level available !\n"
			"Just dive into the code and add yourself more levels, it's easy..." );
		return;
		}

	/*--- Show the level ---*/
	setString( ABW_phreshold_level, "Level %d", level + 1 );

	/*--- Initializations prior to a new play ---*/
	memmove( &currlevel, &conf_levels[ level ], sizeof( CONF_LEVEL ) );

	/*--- Fuel ---*/
	if ( (testing_level != -1) || (level % 4) == 0 )
		{
		fuel.level = 1000;
		fuel.start = clock();
		fuel.nb_clocks = CLOCKS_PER_SEC * currlevel.fuel_time;
		setGauge_MinMax( ABW_phreshold_fuel, 0, fuel.nb_clocks, fuel.nb_clocks );
		}

	/*--- Initialize a new generation of ships ---*/
	init_gen_ships();
	currlevel.nb_ships_alive = currlevel.nbships;

	/*--- Install some stars ---*/
	put_stars();

	/*--- Initialize the game to play level ---*/
	restart_same_level( start_a_cannon );

	/*--- Now the cannon is started, time to play the real game ! ---*/
	refresh_frame( TRUE );

	}	// start_new_level


void start_play( void )
	{
	STARTING_CANNON		*starting_cannon;
	int					n;

	/*--- Initialize score ---*/
	score = 0;
	draw_score();

	/*--- Reset starting cannons ---*/
	nb_cannons_left = NB_STARTING_CANNONS;
	for ( starting_cannon = starting_cannons, n = 0; 
		  n < NB_STARTING_CANNONS; 
		  n++, starting_cannon++ )
		{
		setPos( starting_cannon->wgt, starting_cannon->init_x, starting_cannon->init_y );
		starting_cannon->x = starting_cannon->init_x;
		starting_cannon->y = starting_cannon->init_y;
		}	// for (n, starting_cannon)

	/*--- Start at level #0 ---*/
	if ( testing_level == -1 )
		level = -1;
	else
		level = testing_level;
	start_new_level( TRUE );		// Starting cannon

	}	// start_play


int
menu_file_play( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Nothing to do if already playing ---*/
	if ( is_drawFrame )
		return( Pt_CONTINUE );

	/*--- Just here to avoid that the menu bar has the focus ---*/
	PtContainerGiveFocus( ABW_phreshold_ramp, cbinfo->event );

	/*--- Begin to play ---*/
	start_play();

	return( Pt_CONTINUE );

	}	// menu_file_play
