// ============================================================================================
// ===																						===
// === missile.c																			===
// ===																						===
// === Functions defined:																	===
// ===		launch_missile																	===
// ===		draw_missile																	===
// ===																						===
// ============================================================================================

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "phreshold.h"
#include "abimport.h"
#include "proto.h"


void launch_missile( void )
	{

	/*--- Create the new missile ---*/
	missile.active = TRUE;
	missile.x = cannon.x + ((cannon.w - missile_w) >> 1);
	missile.y = cannon.y;

	/*--- Change the state of the cannon while missile is active ---*/
	cannon.state = CANNON_UNARMED;

	/*--- Update the laser status ---*/
	laser.level += LASER_SHOOT;

	}	// launch_missile


void draw_missile( void )
	{
static	SHIP		*ship;
static	CONF_SHIP	*des;
static	int			n;
static	PhRect_t	rect1, rect2;

	/*--- Non active ? ---*/
	if ( !missile.active )
		return;

	/*--- Draw the missile to the frame buffer ---*/
	blaster_or( missile_img,  missile_w, missile_h,  missile.x, missile.y );

	/*--- Missile disappears ? ---*/
	if ( !missile.y )
		{
		missile.active = FALSE;
		cannon.state = CANNON_ARMED_0;
		cannon.inc = 1;
		return;
		}

	/*--- Move the missile ---*/
	rect2.lr.x = missile.x + missile_w;
	rect2.lr.y = missile.y + missile_h;
	if ( (missile.y -= MISSILE_YINC) < 0 )
		missile.y = 0;
	rect2.ul.x = missile.x;
	rect2.ul.y = missile.y;

	/*--- Any ship hit ? ---*/
	for ( ship = ships, n = 0; n < currlevel.nbships; n++, ship++ )
		{

		des = ship->des;
		if ( !ship->is_hit )
			{
			rect1.ul.x = ship->x;
			rect1.ul.y = ship->y;
			rect1.lr.x = ship->x + des->sz[ ship->state ].w;
			rect1.lr.y = ship->y + des->sz[ ship->state ].h;

			if ( PtRectIntersect( &rect1, &rect2 ) )
				{
				if ( ship->is_attacking )
					score += SCORE_HIT_SHIP_ATTACKING;
				else
					score += SCORE_HIT_SHIP;
				draw_score();
				ship_is_hit( ship );
				missile.active = FALSE;
				cannon.state = CANNON_ARMED_0;
				cannon.inc = 1;
				return;
				}	// if
				
			}	// if

		}	// for (ship, n)
		
	}	// draw_missile
