// ============================================================================================
// ===																						===
// === phreshold.h																			===
// ===																						===
// === 		This file contains all definitions (#define, typedef) used by all modules.		===
// ===		It also contains all the global variables used by all modules. Variables are 	===
// ===		defined into the module 'phreshold.c'.											===
// ===																						===
// ============================================================================================

/*--- __MAIN__ isonly defined into the module phreshold.c ---*/
#ifdef __MAIN__
	#define COMMON
#else
	#define COMMON extern
#endif

/*--- Version number of software ---*/
#define VERSION_NB		0x010000		// version 1.0.A
#define VERSION_DATE	0x000806		// yymmdd

// Options
#define LOGTOTMP		1

/*--- Common things ---*/
typedef unsigned int		UINT;
typedef short int			SHORT;
typedef short unsigned int 	USHORT;
typedef signed char			BYTE;
typedef unsigned char		UBYTE;
typedef signed char			BOOL;
typedef int					IBOOL;
#define FALSE 0
#define TRUE -1

/*--- Configuration file ---*/
#define CFG_FILENAME			"phreshold.ini"
#define CFG_SECTION_WINSZPOS	"Windows Size and Position"
#define	CFG_SECTION_OPTIONS		"Options"

/*--- Memory (shared) allocated for framing ---*/
#define	FRAME_POS_X			20
#define	FRAME_POS_Y			54
// COMMON	UBYTE				*frame;
// #define	FRAME_SHNAME		"shold_frame"
COMMON	UBYTE				*work_frame;
#define	WORK_FRAME_SHNAME	"shold_workfram"
COMMON	UBYTE				*background;

/*--- Size of the frame ---*/
COMMON	int		frame_width;
COMMON	int		frame_height;
COMMON	int		frame_size;
#define FRAME_WIDTH_MIN		480
#define	FRAME_HEIGHT_MIN	360
#define	FRAME_SIZE_MIN		172800		/* FRAME_WIDTH * FRAME_HEIGHT */
#define FRAME_WIDTH_MAX		640
#define	FRAME_HEIGHT_MAX	480
#define	FRAME_SIZE_MAX		307200		/* FRAME_WIDTH * FRAME_HEIGHT */

/*--- Palette used for drawing bitmaps ---*/
COMMON	PgColor_t	palette[ 256 ];
COMMON	long		crc_palette;

/*--- Couple Proxy/Timer signaled to redraw the frame ---*/
COMMON	pid_t	proxy_updateFrame;
COMMON	timer_t	timer_updateFrame;

/*--- PID of the process 'phreshold_kbd' ---*/
COMMON	pid_t	pid_kbd;

/*--- Frame update active ? ---*/
COMMON	IBOOL	is_drawFrame;

/*--- Spec. of the frame ---*/
#ifdef __MAIN__
COMMON	PhPoint_t	frame_pos = { FRAME_POS_X, FRAME_POS_Y };
#else
extern	PhPoint_t	frame_pos;
#endif
COMMON	PhPoint_t	frame_dim;

/*--- Save the palette used ---*/
COMMON	PgColor_t	palette[ 256 ];

/*--- Proxy to be signaled when the cannon is dead ---*/
COMMON	pid_t	proxy_cannonIsDead;


/*--- ----------------- ---*/
/*--- Freeze the game ? ---*/
/*--- ----------------- ---*/

COMMON	IBOOL	is_frozen;
COMMON	time_t	when_frozen;


/*--- --------------- ---*/
/*--- Games constants ---*/
/*--- --------------- ---*/

#define	MAXNB_SHIPS_PER_LEVEL	20
#define	MAXNB_SHIP_STATES		24
#define	MAXNB_SHELLS			72
#define	MAXNB_STARS				12

#define PERIOD	50				// 40 frames/sec i.e. 31.25 msec / frame


/*--- -------------- ---*/
/*--- Testing mode ? ---*/
/*--- -------------- ---*/

COMMON	int	testing_level;		// Set to -1 if not active


/*--- --------------------------------------- ---*/
/*--- Used to create a (temporary) hit bitmap ---*/
/*--- --------------------------------------- ---*/

COMMON	UBYTE	hitbmp[ 8192 ];


/*--- --------------- ---*/
/*--- Starting Cannon ---*/
/*--- --------------- ---*/

#define CANNON_DESTROYED_IN	250

#define	NB_STARTING_CANNONS	5

typedef struct
	{
	PtWidget_t	*wgt;				// Widget
	int			w, h;				// Size (not used, we move the widget with the lib api)
	int			init_x, init_y;		// Initial position
	int			x, 	y;				// Current position
	}	STARTING_CANNON;

COMMON	STARTING_CANNON	starting_cannons[ NB_STARTING_CANNONS ];


/*--- --------------- ---*/
/*--- Ships (enemies) ---*/
/*--- --------------- ---*/

#define MAXNB_BEZIER_CTRL		10		// Max nb of control points on bezier curve
#define	CTRLPTNB_SHIPFORCVIS	10		// Control points

typedef struct
	{
	
	/*--- Pointer to configuration description ---*/
	struct conf_ship	*des;

	/*--- Running state ---*/
	IBOOL	is_dead;					// Ship is dead ?
	int		prev_x, prev_y, prev_state;	// Previous position and state
	int		x, y;						// Current position
	int		state, inc;					// Current state and increment (-1, +1)
	int		idle_cnt;

	/*--- Moving to this new position ---*/
	int		move_dx, move_dy;			// Offset
	int		cnt_movex, cnt_movey;

	/*--- Attacking ---*/
	IBOOL		is_attacking;					// Ship is now attacking ?
	IBOOL		is_downCalcAttacking;			// Computing points for next attack (down)
	int			downCtrlPtNb;					// Nb of ctrl point
	int			downPtNb, downPtIndex;			// Points nb and index
	PhPoint_t	downPt[MAXNB_BEZIER_CTRL+2];	// Each	position of	control	points (relativly to the ship)
	IBOOL		is_upCalcAttacking;				// Computing points for next attack (down)
	int			upCtrlPtNb;						// Nb of ctrl point
	int			upPtNb, upPtIndex;				// Points nb and index
	PhPoint_t	upPt[MAXNB_BEZIER_CTRL+2];		// Each	position of	control	points (relativly to the ship)

	/*--- Back to be visible ---*/
	time_t		when_nonvisible;				// Time 1st time non visible
	IBOOL		is_nonvisible;					// Is the ship now visible ?
	IBOOL		forcing_visible;				// Ship is now descending (to be not hidden) ?
	int			forcVisPtNb, forcVisPtIndex;	// Points nb and index
	PhPoint_t	forcVisPt[ CTRLPTNB_SHIPFORCVIS+2 ];	// Control points

	/*--- Ship is hit - Destruction process ---*/
	IBOOL		is_hit;							// Ship is hit
	int			hitcnt;							// Counter, down to 0

	}	SHIP;

COMMON	SHIP	ships[ MAXNB_SHIPS_PER_LEVEL ];


/*--- ------ ---*/
/*--- Shells ---*/
/*--- ------ ---*/

// Current nb of active shells
COMMON	int		nb_active_shells;

// Description of a shell
typedef struct
	{
	IBOOL	active;				// Is this shell active ?
	int		x, y;				// Current position
	int		incy;				// Speed
	}	SHELL;

// Pool of all shells (active or not)
COMMON	SHELL	shells[ MAXNB_SHELLS ];

// Description of the bitmap
COMMON	PhImage_t	*shell_img_des;
COMMON	UBYTE		*shell_img;
COMMON	int			shell_w, shell_h;


/*--- ------ ---*/
/*--- Cannon ---*/
/*--- ------ ---*/

// Description of the cannon
typedef struct
	{
	int		x, y;				// Current position
	int		cnt;				// 
	int		state;				// CANNON_ARMED_0-2 | CANNON_UNARMED
	int		inc;
	IBOOL	is_hit;				// Cannon is hit ?
	int		hitcnt;				// Counter, down to 0
	int		w, h;
	}	CANNON;

// The cannon
COMMON	CANNON	cannon;

// Description of the bitmap
#define CANNON_ARMED_0	0
#define CANNON_ARMED_1	1
#define CANNON_ARMED_2	2
#define	CANNON_UNARMED	3
#define CANNON_DEAD		4
COMMON	PhImage_t	*cannon_img_des[ 5 ];
COMMON	UBYTE		*cannon_img[ 5 ];

// State of the left/right keys
COMMON	IBOOL	is_left, is_right;

// Speed of the cannon
#define CANNON_XINC		6


/*--- ------- ---*/
/*--- MISSILE ---*/
/*--- ------- ---*/

// Description of the Missile
typedef struct
	{
	int		x, y;				// Current position
	IBOOL	active;				// Active or not
	}	MISSILE;

// The Missile
COMMON	MISSILE	missile;

// Description of the bitmap
COMMON	PhImage_t	*missile_img_des;
COMMON	UBYTE		*missile_img;
COMMON	int			missile_w, missile_h;

// Missile speed
#define	MISSILE_YINC	12


/*--- ---------- ---*/
/*--- LASER HOT ---*/
/*--- ---------- ---*/

#define	LASER_SHOOT		70
#define	LASER_OVERHEAT	900
#define	LASER_OK		650
#define	LASER_DEC		35

typedef struct
	{
	double		level;			// Current level, from 0 to 1000
	IBOOL		overheat;		// Is laser overheat ?
	clock_t		prev_clock;
	double		per_clock;
	}	LASER;

COMMON	LASER	laser;


/*--- ---------- ---*/
/*--- FUEL LEVEL ---*/
/*--- ---------- ---*/

typedef struct
	{
	int			level;			// Current level, from 1000 to 0
	clock_t		start;			// Starting time
	int			nb_clocks;		// Nb of seconds given at startup
	}	FUEL;

COMMON	FUEL	fuel;


/*--- ------ ---*/
/*--- SCORES ---*/
/*--- ------ ---*/

#define	SCORE_HIT_SHIP				100
#define	SCORE_HIT_SHIP_ATTACKING	200


/*--- ------------------- ---*/
/*--- Level configuration ---*/
/*--- ------------------- ---*/

// Function to place ships
typedef	void (*POS_FUNCT) (struct conf_ship *des);

typedef struct conf_ship
	{

	/*--- Global parameters ---*/
	int			ship_fileno;			// No of the bitmap
	IBOOL		moving_grouped;			// Do the ships move grouped ?
	IBOOL		scroll_lr, scroll_bt;	// Scrolling left-right, Scrolling bottom-top
	IBOOL		same_state;				// Do ships have the same state ?
	POS_FUNCT	pos_funct;				// Function to place ships
	int			nbships1, nbships2;		// Nb of ships (btw [n1..n2])
	int			x1, x2, xn1, xn2;		// Generate a random X movement btw [x1, x2]
	int			y1, y2, yn1, yn2;		// Generate a random X movement btw [x1, x2]
	int			bottom_ymax;			// Bottom gap btw the bottom and ship

	/*--- Description of the ships ---*/
	IBOOL		rounding;					// State transition: 01232101232... or 012301230...
	int			nb_states;					// Nb of states of each ship
	PhPoint_t	offs[ MAXNB_SHIP_STATES ];	// Offset (transition btws each state)
	int			maxnb_sec_hidden;			// Max nb of sec. of ship being hidden

	/*--- Attack ---*/
	int			agressivity;			// Level of agressivity (shells drop)
	int			drops1, drops2;			// Speed of shells (between)
	int			freq_attack;			// Frequency of the attacks

	/*--- This part is filled at run-time ---*/
	int			nbships;						// Nb of ships (initial)
	PhImage_t	*img_des[ MAXNB_SHIP_STATES ];
	UBYTE		*img[ MAXNB_SHIP_STATES ];
	PhDim_t		sz[ MAXNB_SHIP_STATES ];		// Size of the ships (for each state)
	}	CONF_SHIP;

typedef struct conf_level
	{

	/*--- Nb of different ships ---*/
	int			nbdiff_ships;

	/*--- Ship decription ---*/
	int			fuel_time;				// Fuel time (in seconds)
	int			max_nb_attacks;			// Max nb of ships attacking at same time
	int			maxnb_shells;			// Max number of shells for this level
	CONF_SHIP	shipdes[ 4 ];

	/*--- This part is filled at run-time ---*/
	int			nbships;					// Nb of ships (initial)
	int			nb_ships_attacking;			// Nb of ships attacking now
	int			nb_ships_alive;				// Nb of ships still alive

	}	CONF_LEVEL;

COMMON	int	nb_levels;

COMMON	int	level;
COMMON	int	score;
COMMON	int	nb_cannons_left;

// Used by the low-level function blaster_or
COMMON	int	is_scrolling_lr;
COMMON	int	is_scrolling_bt;

COMMON	IBOOL	is_starting_cannon;


/*--- ------ ---*/
/*--- Mother ---*/
/*--- ------ ---*/

// Description of the mother ship
typedef struct
	{
	int		x, y;				// Current position
	int		w, h;
	}	MOTHER;

// The mother ship
COMMON	MOTHER		mother;

// Description of the bitmap
COMMON	PhImage_t	*mother_img_des;
COMMON	UBYTE		*mother_img;


/*--- ----- ---*/
/*--- Stars ---*/
/*--- ----- ---*/

// Description of a star
typedef struct
	{
	int		x, y;				// Current position
	int		state, inc;
	int		speed, speed_cnt;
	}	STAR;

// The stars
COMMON	STAR		stars[ MAXNB_STARS ];
COMMON	int			nb_stars;
#define	STAR_NBSTATES	6
#define	STAR_HEIGHT		9
#define	STAR_WIDTH		9

// Description of the bitmap
COMMON	PhImage_t	*star_img_des[ STAR_NBSTATES ];
COMMON	UBYTE		*star_img[ STAR_NBSTATES ];


/*--- ------------------------------- ---*/
/*--- Create the configuration levels ---*/
/*--- ------------------------------- ---*/

COMMON	CONF_LEVEL	currlevel;
#ifdef __MAIN__

#include "levels.h"

#else

extern	CONF_LEVEL	conf_levels[];

#endif


/*--- ----------------- ---*/
/*--- Blaster functions ---*/
/*--- ----------------- ---*/

#pragma aux blaster_copy parm [esi][eax][ebx][ecx][edx];
#pragma aux blaster_xor  parm [esi][eax][ebx][ecx][edx];
#pragma aux blaster_or   parm [esi][eax][ebx][ecx][edx];
#define BLASTER( OP ) \
	void blaster_##OP( const UBYTE *src,  \
					   const int dw, const int dh, \
					   const int px, const int py )
BLASTER( copy );
BLASTER( or );
BLASTER( xor );


/*--- --------------- ---*/
/*--- Random function ---*/
/*--- --------------- ---*/

#pragma aux rnd parm [ebx][ecx] value [edx];
extern int rnd( const int n1, const int n2 );


/*--- .ini filename ---*/
COMMON	char	ini_fname[ PATH_MAX+30];

