// ============================================================================================
// ===																						===
// === phreshold_kbd.c																		===
// ===																						===
// ===		This process is created by the process 'phreshold'. Its purpose is to draw		===
// ===		periodically (say 25Hz) the frame (shared memory).								===
// ===																						===
// ============================================================================================

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <limits.h>
#include <time.h>
#include <sys/proxy.h>
#include <sys/kernel.h>
#include <sys/psinfo.h>
#include <sys/osinfo.h>
#include <sys/sched.h>

#include <Pt.h>

/*--- Common definitions shared by the two tasks: phreshold and phreshold_kbd ---*/
#include "phreshold_kbd.h"


static	KEYS_SHARED		*keys_state;

static	pid_t			pid_main;

/*--- Couple Proxy/Timer signaled at a slow rate to check if main process is there ---*/
static	pid_t			proxy_checkMain;
static	timer_t			timer_checkMain;


static void do_exit( void )
	{

	/*--- Unlink the shared memories ---*/
	shm_unlink( KEYS_SHNAME );

	/*--- Exit this task ---*/
	exit( 0 );

	}	// do_exit


void checkMain( void )
	{
	struct _psinfo	psdata;

	if ( (getppid() != pid_main) ||
		 (qnx_psinfo( PROC_PID, getppid(), &psdata, 0, NULL ) != getppid()) ||
		 (psdata.pid != getppid()) )
		do_exit();

	}	// checkMain


static PhRid_t open_region( void )
	{
	PhRegion_t	region;
	PhRid_t		rid;
	PhRect_t	rect;
	
	memset( &region, 0, sizeof( PhRegion_t ) );
	region.events_sense  = Ph_EV_KEY;
	region.events_opaque = 0;
	region.parent = Ph_ROOT_RID;

	rect.ul.x = rect.ul.y = 0;
	rect.lr.x = 1279;
	rect.lr.y = 1023;
	
	rid = PhRegionOpen( Ph_REGION_PARENT | Ph_REGION_EV_SENSE | Ph_REGION_EV_OPAQUE |
		Ph_REGION_ORIGIN | Ph_REGION_RECT,
		&region, &rect, NULL );

	return rid;
	
	}	// open_region


static void key( PhKeyEvent_t *key_event )
	{
	int		key_down;
	
	key_down = PkIsKeyDown( key_event->key_flags );
	switch ( key_event->key_cap )
		{

		case Pk_Control_L	:
			keys_state->shoot = key_down;
			break;

		case Pk_Left	: 
			keys_state->left = key_down;
			break;

		case Pk_Right	: 
			keys_state->right = key_down;
			break;

		case Pk_Escape	:
			keys_state->escape = 1;
			keys_state->escape_down = key_down;
			break;

		case Pk_X :
		case Pk_x : 
			keys_state->exit = 1;
			break;

		}	// switch

	}	// key


void main( int argc, char *argv[] )
	{
	struct sigevent		sigevnt;
	struct itimerspec	timerSpec;
	PhEvent_t			*event;
	pid_t				pid;

	/*--- Get pid form the father process ---*/
	pid_main = getppid();

	/*--- Photon library initializations ---*/
	PtInit( NULL );
//	PhAttach( NULL, NULL );

	/*--- Create shared memory used to control the use of the frame (semaphore) ---*/
	if ( (keys_state = PgShmemCreate( sizeof( KEYS_SHARED ), KEYS_SHNAME )) == NULL )
		{
		if ( (shm_unlink( KEYS_SHNAME )) == -1 )
			{
			fprintf( stderr, "Unable to unlink shared memory '%s' [err=%d]\n",
				KEYS_SHNAME, errno );
			exit( -1 );
			}
		if ( (keys_state = PgShmemCreate( sizeof( KEYS_SHARED ), KEYS_SHNAME )) == NULL )
			{
			fprintf( stderr, "Unable to create shared memory '%s' [err=%d]\n",
				KEYS_SHNAME, errno );
			exit( -1 );
			}
		}	// if
	keys_state->left   =
	keys_state->right  =
	keys_state->shoot  = 
	keys_state->escape = keys_state->escape_down =
	keys_state->exit   = 0;

	/*--- Couple Proxy/Timer signaled at a slow rate to check if main process is there ---*/
	if ( (proxy_checkMain = qnx_proxy_attach( 0, 0, 0, -1 )) == -1 )
		{
		fprintf( stderr, "No more proxy available.\n" );
		exit( -1 );
		}
	sigevnt.sigev_signo = -proxy_checkMain;
	if ( (timer_checkMain = timer_create( CLOCK_REALTIME, &sigevnt )) == -1 ) 
		{
		fprintf( stderr, "No more timer available.\n" );
		exit( -1 );
		}
	timerSpec.it_value.tv_sec     = 1;			
	timerSpec.it_value.tv_nsec    = 0;
	timerSpec.it_interval.tv_sec  = 1;
	timerSpec.it_interval.tv_nsec = 0;
	timer_settime( timer_checkMain, 0, &timerSpec, NULL );

	open_region();

	/*--- Set priority ---*/
	setprio( 0, 11 );

	/*--- Loop until user quits application ---*/
	if ( (event = malloc( sizeof( PhEvent_t ) + 1000 )) == NULL )
		{
		fprintf( stderr, "No memory !\n" );
		exit( -1 );
		}
	PhEventArm();
	for (;;)
		{
		pid = Receive( 0, NULL, 0 );
		if ( PhEventRead( pid, event, sizeof( PhEvent_t ) + 1000 ) == Ph_EVENT_MSG )
			{
			if ( event->type == Ph_EV_KEY )
				key( (PhKeyEvent_t *)PhGetData( event ) );
			}
		else
			if ( pid == proxy_checkMain )
				checkMain();
		}	// for (;;)
	
	}	// main
