// ============================================================================================
// ===																						===
// === shells.c																				===
// ===																						===
// === Functions defined:																	===
// ===		drop_shell																		===
// ===		move_shells																		===
// ===																						===
// ============================================================================================

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "phreshold.h"
#include "abimport.h"
#include "proto.h"


void drop_shell( const SHIP *ship )
	{
static	SHELL		*shell;
static	CONF_SHIP	*des;
static	int			n;

	/*--- No room for a new shell ? ---*/
	if ( nb_active_shells == currlevel.maxnb_shells )
		return;

	/*--- This level is finished ? ---*/
	if ( !currlevel.nb_ships_alive )
		return;

	/*--- Found an empty slot ---*/
	for ( shell = shells, n = 0; n < currlevel.maxnb_shells; n++, shell++ )
		if ( !shell->active )
			break;

	/*--- Create the shell ---*/
	des = ship->des;
	shell->y = ship->y + des->sz[ ship->state ].h;
	shell->x = ship->x + ((des->sz[ ship->state ].w - shell_w) >> 1);
	shell->incy = rnd( des->drops1, des->drops2 );
	shell->active = TRUE;
	nb_active_shells++;
	
	}	// drop_shell


void move_shells( void )
	{
static	SHELL	*shell;
static	int		n;

	/*--- This level is finished ? ---*/
	if ( !currlevel.nb_ships_alive && !nb_active_shells )
		{
		if ( testing_level == -1 )
			start_new_level( FALSE );
		else
			refresh_frame( FALSE );
		return;
		}

	/*--- Move all active shells ---*/
	for ( shell = shells, n = 0; n < currlevel.maxnb_shells; n++, shell++ )
		{

		/*--- Not active ? ---*/
		if ( !shell->active )
			continue;

		/*--- Shell to be lost ? ---*/
		if ( shell->y == frame_height - shell_h )
			{
			shell->active = FALSE;
			nb_active_shells--;
			continue;
			}

		/*--- Draw the shell to the frame buffer ---*/
		blaster_or( shell_img, 	shell_w, shell_h,  shell->x, shell->y );

		/*--- Compute new position of the shell ---*/
		if ( (shell->y += shell->incy) > (frame_height - shell_h) )
			shell->y = frame_height - shell_h;

		/*--- Test if shell hit the cannon ---*/
		if ( shell->y < (frame_height - cannon.h) )
			continue;
		if ( (shell->x >= cannon.x) && (shell->x + shell_w <= cannon.x + cannon.w) )
			{
			cannon_is_hit();
			shell->active = FALSE;
			nb_active_shells--;
			}

		}	// for (shell, n)
	
	}	// move_shells
