/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <limits.h>
#include <sys/kernel.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "phreshold.h"
#include "abimport.h"
#include "proto.h"

static	int			cnt;


/*--- 0 means BLACK i.e. transparent... ---*/
void make_hit_bitmap( const UBYTE *src, const int sz, const int nb_rem )
	{
			int		n, z;
register	int		k;
			int		nb_set;

	if ( nb_rem < sz/2 )
		{
		memmove( hitbmp, src, sz );
		for ( n = 0; n < nb_rem; n++ )
			{
			for ( k = 0; k < 1; k++ )
				if ( hitbmp[ z = rnd( 0, sz-1 ) ] != 0 )
					break;
			hitbmp[ z ] = 0;
			}
		}
	else
		{
		memset( hitbmp, 0, sz );
		nb_set = sz - nb_rem;
		for ( n = 0; n < nb_set; n++ )
			{
			for ( k = 0; k < 3; k++ )
				if ( hitbmp[ z = rnd( 0, sz-1 ) ] == 0 )
					break;
			hitbmp[ z ] = src[ z ];
			}
		}
	
	}	// make_hit_bitmap


/*--- ------------------------------------ ---*/
/*--- Initialize a new generation of ships ---*/
/*--- ------------------------------------ ---*/

void init_gen_ships( void )
	{
	SHIP		*ship;
	int			w, n;
	char		fname[ 40 ];
	int			set, cpt;
	CONF_SHIP	*des;

	/*--- ---------------- ---*/
	/*--- Load all bitmaps ---*/
	/*--- ---------------- ---*/

	for ( w = 0; w < currlevel.nbdiff_ships; w++ )
		for ( n = 0; n < currlevel.shipdes[ w ].nb_states; n++ )
			{

			sprintf( fname, "L%d.%d.bmp", currlevel.shipdes[ w ].ship_fileno, n );
			currlevel.shipdes[ w ].img_des[ n ] = load_bitmap( fname );
			currlevel.shipdes[ w ].img[ n ]  = currlevel.shipdes[ w ].img_des[ n ]->image;
			currlevel.shipdes[ w ].sz[ n ].w = currlevel.shipdes[ w ].img_des[ n ]->size.w;
			currlevel.shipdes[ w ].sz[ n ].h = currlevel.shipdes[ w ].img_des[ n ]->size.h;
	
			}	// for (n)

	
	/*--- ---------------- ---*/
	/*--- How many ships ? ---*/
	/*--- ---------------- ---*/

	currlevel.nbships = 0;	
	for ( w = 0; w < currlevel.nbdiff_ships; w++ )
		{
		currlevel.shipdes[ w ].nbships = 
			rnd( currlevel.shipdes[ w ].nbships1, currlevel.shipdes[ w ].nbships2 );
		currlevel.nbships += currlevel.shipdes[ w ].nbships;
		}


	/*--- ---------------------------------- ---*/
	/*--- Found a position for all the ships ---*/	
	/*--- ---------------------------------- ---*/

	memset( &ships, 0, sizeof( ships ) );	
	for ( ship = ships, cpt = 0, set = 0, n = 0; n < currlevel.nbships; n++, ship++ )
		{

		/*--- Which ship set ? ---*/
		if ( cpt++ == currlevel.shipdes[ set ].nbships )
			{
			set++;
			cpt = 0;
			}
		ship->des = des = &currlevel.shipdes[ set ];

		/*--- Ship state ---*/
		ship->is_dead = FALSE;
		ship->is_hit  = FALSE;
		ship->hitcnt  = 0;
		ship->when_nonvisible = 0;

		/*--- Create an initial state ---*/
		if ( !des->same_state || !n )
			{
			if ( des->nb_states == 1 )
				{
				ship->state = 0;
				ship->inc	= 0;
				}
			else if ( des->nb_states == 2 )
				{
				ship->state = rnd( 0, 1 );
				ship->inc	= 1 - 2*ship->state;
				}
			else
				{
				ship->state = rnd( 1, des->nb_states - 2 );
				ship->inc	= 2*rnd( 0, 1) - 1;
				}
			}	// if
		if ( des->same_state && n )
			{
			ship->state = ships[ 0 ].state;
			ship->inc   = ships[ 0 ].inc;
			}

		/*--- Found a position for this ship ---*/
		if ( des->pos_funct == NULL )
			{
			ship->x = rnd( 1, frame_width-1 );
			ship->y = rnd( 1, frame_height - des->bottom_ymax - 1 );
			}

		}	// for (ship, n)

	/*--- Function to position all ships ? ---*/
	for ( set = 0; set < currlevel.nbdiff_ships; set++ )
		if ( currlevel.shipdes[ set ].pos_funct )
			currlevel.shipdes[ set ].pos_funct( &currlevel.shipdes[ set ] );

	cnt = 0;

	}	// init_gen_ships


void is_cannon_hit_by_ship( SHIP *ship )
	{
static	PhRect_t	rect1, rect2;

	/*--- ------------------------------------------------ ---*/
	/*--- Check now if the new ship has not hit the cannon ---*/
	/*--- ------------------------------------------------ ---*/

	if ( !ship->is_hit && !cannon.is_hit && !is_starting_cannon && !testing_level )
		{

		/*--- The cannon area ---*/
		rect2.ul.x = cannon.x;
		rect2.ul.y = cannon.y;
		rect2.lr.x = cannon.x + cannon.w;
		rect2.lr.y = cannon.y + cannon.h;

		/*--- The ship area ---*/	
		rect1.ul.x = ship->x;
		rect1.ul.y = ship->y;
		rect1.lr.x = ship->x + ship->des->sz[ ship->state ].w;
		rect1.lr.y = ship->y + ship->des->sz[ ship->state ].h;

		if ( PtRectIntersect( &rect1, &rect2 ) )
			{
	
			/*--- Cannon is dead ---*/
			cannon_is_hit();

			/*--- But also the ship... ---*/
			if ( ship->is_attacking )
				score += SCORE_HIT_SHIP_ATTACKING;
			else
				score += SCORE_HIT_SHIP;
			draw_score();
			ship_is_hit( ship );
			}
			
		}	// if

	}	// is_cannon_hit_by_ship


static void start_new_attack( SHIP *ship )
	{
	int			n;
	int			nCtrlPtNb;
	PhPoint_t	*q;					// Control points

	ship->is_attacking			= TRUE;			// Attack in progress
	ship->is_downCalcAttacking	= TRUE;			// Computing points for next attack (down)
	ship->is_upCalcAttacking	= FALSE;		// Computing points for next attack (down)

	/*----------------*/
	/*--- (1) Down ---*/
	/*----------------*/

	/*--- Generate points of control of the bezier curve ---*/
	q = ship->downPt;
	q->x = ship->x;
	q->y = ship->y;
	nCtrlPtNb = rnd( 3, 9 );
	for ( q++, n = 0; n < nCtrlPtNb; n++, q++ )
		{
		q->x = rnd( -(frame_width  >> 1), (3 * frame_width)  >> 1 );
		q->y = rnd( -(frame_height >> 1), (3 * frame_height) >> 2 );
		}	// for (n)
	q->x = cannon.x + (cannon.w >> 1);
	q->y = cannon.y + (cannon.h >> 1);
 
	/*--- Number of	points in the curve	---*/
	ship->downPtNb     = rnd( 3, 6 ) * 40;
    ship->downPtIndex  = 0;
	ship->downCtrlPtNb = nCtrlPtNb;


	/*--------------*/
	/*--- (2) Up ---*/
	/*--------------*/
 
	/*--- Generate points of control of the bezier curve ---*/
	q = ship->upPt;
	q->x = cannon.x + (cannon.w >> 1);
	q->y = cannon.y + (cannon.h >> 1);
	nCtrlPtNb = rnd( 3, 9 );
	for ( q++, n = 0; n < nCtrlPtNb; n++, q++ )
		{
		q->x = rnd( -(frame_width  >> 1), (3 * frame_width)  >> 1 );
		q->y = rnd( -(frame_height >> 1), (3 * frame_height) >> 2 );
		}	// for (n)
	q->x = ship->x;
	q->y = ship->y;
 
	/*--- Number of	points in the curve	---*/
	ship->upPtNb     = rnd( 3, 6 ) * 40;
	ship->upCtrlPtNb = nCtrlPtNb;

	/*--- Nb of ships attacking now ---*/
	currlevel.nb_ships_attacking++;

	}	// start_new_attack


static void ship_attacking( SHIP *ship )
	{
	double		x, y;							// Points computed

	/*---------------------------------------------*/
	/*--- Is the ship attacking ? (down to gun) ---*/
	/*---------------------------------------------*/

	if ( ship->is_downCalcAttacking )
		{

		/*--- End of the curve ? ---*/
		if (ship->downPtNb == ship->downPtIndex)
			{

			ship->is_downCalcAttacking = FALSE;	// Computing points for next attack (down)
			ship->is_upCalcAttacking   = TRUE;		// Computing points for next attack (up)
			ship->upPtIndex		   = 0;		// Index

			} // if
		else
			{

			Bezier(&x, &y, ship->downPtIndex++ / (double)ship->downPtNb, ship->downCtrlPtNb + 1, ship->downPt);
			ship->x = (int)x;
			ship->y = (int)y;

			} // else

		} // if (ship->is_downCalcAttacking)


	/*-------------------------------------------*/
	/*--- Is the ship attacking ? (up to pos) ---*/
	/*-------------------------------------------*/

	if (ship->is_upCalcAttacking)
		{

		/*--- End of the curve ? ---*/
		if (ship->upPtNb == ship->upPtIndex)
			{

			ship->is_upCalcAttacking = FALSE;	// Computing points for next attack (down)
			ship->is_attacking		 = FALSE;	// End of attack
			ship->forcing_visible	 = FALSE;

			/*--- Update attacking ships nb ---*/
			currlevel.nb_ships_attacking--;

			} // if
		else
			{

			Bezier(&x, &y, ship->upPtIndex++ / (double)ship->upPtNb, ship->upCtrlPtNb + 1, ship->upPt);
			ship->x = (int)x;
			ship->y = (int)y;

			} // else

		} // if (ship->is_upCalcAttacking)

	/*--- Does the ship hit the cannon ? ---*/
	is_cannon_hit_by_ship( ship );

	} // ShipAttacking


static void force_ship_to_be_visible( SHIP *ship )
	{
	PhPoint_t	*p;				// Pointer into ctrl point table
	int			 n;				// Loop	index
	double		 dx, dy;		// Distance in X-Y
	int			 dist;			// The distance between source and destination

	/*--- The original starting point ---*/
	p = ship->forcVisPt;
	p[ 0 ].x = ship->x;
	p[ 0 ].y = ship->y;

	/*--- Define control points	of bezier curve	---*/
	for ( n = 1; n <= CTRLPTNB_SHIPFORCVIS; n++ )
		{
		p[ n ].x = rnd( 1,	frame_width );
		p[ n ].y = rnd( 1,	frame_height );
		} // for (n)

	/*--- Where the ship go ---*/
	p[ n ].x = rnd( 1, frame_width - 1 );
	p[ n ].y = rnd( 1, frame_height >> 1 );

	/*--- The distance between source and destination ---*/
	dx   = fabs( p[ n ].x - p[ 0 ].x);
	dy   = fabs( p[ n ].y - p[ 0 ].y);
	dist = (int)sqrt( dx*dx + dy*dy );

	/*--- Number of	points in the curve	---*/
	ship->forcVisPtNb     = (int)(dist / (rnd( 75, 150 ) / 100.0));
    ship->forcVisPtIndex  = 0;
	ship->forcing_visible = TRUE;			// To be not hidden

	/*--- Does the ship hit the cannon ? ---*/
	is_cannon_hit_by_ship( ship );

	}	// force_ship_to_be_visible


static void force_ship_visible( SHIP *ship )
	{
	double		x, y;					// Points computed

	/*--- End of the curve ? ---*/
    if ( ship->forcVisPtIndex >= ship->forcVisPtNb )
		{

		ship->forcing_visible	= FALSE;

		} // if
	else
		{

		Bezier( &x, &y, ship->forcVisPtIndex++ / (double)ship->forcVisPtNb, 
			CTRLPTNB_SHIPFORCVIS + 1, ship->forcVisPt);
		ship->x = (int)x;
		ship->y = (int)y;

		} // else

	}	// force_ship_visible
	

static void new_position_ship( SHIP *ship, const SHIP *first_ship, const int ydec )
	{
	int			yinc;
	int			cnt;
	CONF_SHIP	*des;

	/*--- Compute new position of the ship ---*/
	des = ship->des;
	if ( !des->moving_grouped || (ship == first_ship) )
		{
		if ( !ship->cnt_movex-- )
			{
			do
				ship->cnt_movex = rnd( des->xn1, des->xn2 );
			while ( !ship->cnt_movex );
			ship->move_dx = rnd( des->x1, des->x2 ) / ship->cnt_movex;
			}
		if ( !ship->cnt_movey-- )
			{
			do
				ship->cnt_movey = rnd( des->yn1, des->yn2 );
			while ( !ship->cnt_movey );
			if ( !des->scroll_bt )
				{
				cnt = 0;
				do
					ship->move_dy = ( yinc = rnd(des->y1, des->y2 )) / ship->cnt_movey;
				while
					((++cnt != 10) && ship->y + yinc > (frame_height - des->bottom_ymax) );
				if ( cnt == 10 )
					ship->move_dy = ((frame_height - des->bottom_ymax) - ship->y);
				}
			else
				ship->move_dy = ( rnd(des->y1, des->y2 ) ) / ship->cnt_movey;
			}
		ship->x += ship->move_dx;
		ship->y += ship->move_dy;
		}
	else
		{
		ship->x += first_ship->move_dx;
		if ( ydec != INT_MAX )
			ship->y += first_ship->move_dy;
		else
			ship->y -= ydec;
		}

	/*--- In case of scrolling ---*/
	if ( des->scroll_lr )
		{
		if ( ship->x + des->sz[ ship->state ].w < 0 )
			ship->x = frame_width - des->sz[ ship->state ].w;
		else
			ship->x %= frame_width;
		}
	if ( des->scroll_bt )
		{
		ship->y %= frame_height;
		ship->y = abs( ship->y );
		}

	/*--- Does the ship hit the cannon ? ---*/
	is_cannon_hit_by_ship( ship );

	}	// new_position_ship


/*--- --------------------- ---*/
/*--- Move the ships armada ---*/
/*--- --------------------- ---*/

void move_ships( void )
	{
static	SHIP		*ship;
static	CONF_SHIP	*des;
static	int			n;
		SHIP		*first_ship;
		int			ydec;
		time_t		currtime;
		IBOOL		change_state;

	/*--- Ships too low ? ---*/
	{
	int	bottom_ymax;
	for ( ydec = INT_MIN, ship = ships, n = 0; n < currlevel.nbships; n++, ship++ )
		{
		bottom_ymax = frame_height - ship->des->bottom_ymax;
		if ( ship->y > bottom_ymax )
			if ( ship->y - bottom_ymax > ydec )
				ydec = (ship->y - bottom_ymax);
		}
	}

	change_state = ((cnt++ % 2) == 0);

	/*--- For each ship ---*/
	currtime = time( NULL );
	for ( first_ship = NULL, ship = ships, n = 0; n < currlevel.nbships; n++, ship++ )
		{

		/*--- ----------------------------- ---*/
		/*--- Nothing to do if ship is dead ---*/
		/*--- ----------------------------- ---*/

		des = ship->des;
		if ( ship->is_dead )
			continue;
		if ( !first_ship )
			first_ship = ship;
			

		/*--- ------------ ---*/
		/*--- New position ---*/
		/*--- ------------ ---*/

		if ( change_state )
		if ( ship->is_attacking )
			{
			ship_attacking( ship );
			}
		else if ( ship->forcing_visible )
			{
			force_ship_visible( ship );
			}
		else
			{
			IBOOL	skip = FALSE;
			if ( des->maxnb_sec_hidden )
				{
				ship->is_nonvisible = (ship->x < 0) || (ship->x >= frame_width) ||
					(ship->y < 0) || (ship->y >= frame_height);
				if ( !ship->is_nonvisible )
					ship->when_nonvisible = 0;
				else
					if ( !ship->when_nonvisible )
						ship->when_nonvisible = currtime;

				if ( ship->is_nonvisible && 
					((currtime - ship->when_nonvisible) >= des->maxnb_sec_hidden) )
					{
					force_ship_to_be_visible( ship );
					skip = TRUE;
					}
				}
				if ( !skip )
				new_position_ship( ship, first_ship, ydec );
			}


		/*--- ---------------------------------------- ---*/
		/*--- Copy the ship bitmap to the frame buffer ---*/
		/*--- ---------------------------------------- ---*/

		/*--- Used by the low-level function blaster_or ---*/
		is_scrolling_lr = des->scroll_lr;
		is_scrolling_bt = des->scroll_bt;

		if ( !ship->is_hit )		
			blaster_or( des->img[ ship->state ], 
				des->sz[ ship->state ].w, des->sz[ ship->state ].h,
				ship->x, ship->y );
		else
			{
			static int sz;
			sz = (des->sz[ ship->state ].w * des->sz[ ship->state ].h);
			make_hit_bitmap( des->img[ ship->state ],
				sz, sz - (sz * --ship->hitcnt) / 150 );
			blaster_or( hitbmp,
				des->sz[ ship->state ].w, des->sz[ ship->state ].h,
				ship->x, ship->y );
			}


		if ( !change_state && !ship->is_attacking )
			continue;


		/*--- --------------------------------- ---*/
		/*--- Compute the new state of the ship ---*/
		/*--- --------------------------------- ---*/

		if ( (des->nb_states > 1) && (!ship->is_hit) )
			{
			ship->prev_state = ship->state;
			if ( des->rounding )
				{
				if ( ship->inc == 1 )
					{
					if ( ++ship->state == des->nb_states )
						{
						ship->state -= 2;
						ship->inc = -1;
						}
					}
				else
					{
					if ( !ship->state-- )
						{
						ship->state += 2;
						ship->inc = 1;
						}
					}
				}	// if
			else
				{
				ship->state = ((ship->state + 1) % des->nb_states);
				}
			}	// if


		/*--- --------------------------- ---*/
		/*--- Adjust position of the ship ---*/
		/*--- --------------------------- ---*/

		if ( (des->nb_states > 1) && (!ship->is_hit) )
			{
			if ( !ship->state && (ship->prev_state == (des->nb_states-1)) )
				{
				ship->x += des->offs[ 0 ].x;
				ship->y += des->offs[ 0 ].y;
				}
			else
				if ( ship->state > ship->prev_state )
					{
					ship->x += des->offs[ ship->state ].x;
					ship->y += des->offs[ ship->state ].y;
					}
				else
					{
					ship->x -= des->offs[ ship->prev_state ].x;
					ship->y -= des->offs[ ship->prev_state ].y;
					}
			}	// if


		/*--- -------------- ---*/
		/*--- Ship is dead ? ---*/
		/*--- -------------- ---*/
		
		if ( ship->is_hit )
			{
			if ( !--ship->hitcnt )
				{
				ship->is_dead = TRUE;
				if ( !--currlevel.nb_ships_alive && !nb_cannons_left && cannon.is_hit )
					{
					refresh_frame( FALSE );
					setString( ABW_phreshold_msg, 
						"Sorry, you loose !\nAll the galaxy curse your name now..." );
					break;
					}
				}	// if
			ship->prev_state = ship->state;		
			continue;
			}	// if


		/*--- -------------------------- ---*/
		/*--- Time to drop a new shell ? ---*/
		/*--- -------------------------- ---*/

		if ( !cannon.is_hit && !is_starting_cannon )
			if ( (ship->x >= 0) && (ship->x < frame_width) )
				if ( !rnd( 0, des->agressivity ) )
					drop_shell( ship );


		/*--- ---------------------------- ---*/
		/*--- Time to start a new attack ? ---*/
		/*--- ---------------------------- ---*/

		if ( !ship->forcing_visible && !cannon.is_hit && !is_starting_cannon )
			if ( currlevel.nb_ships_attacking < currlevel.max_nb_attacks )
				if ( !rnd( 0, des->freq_attack ) )
					start_new_attack( ship );


		ship->prev_state = ship->state;		

		}	// for (ship, n)

	}	// move_ships


void ship_is_hit( SHIP *ship )
	{

	/*--- Update internal state of the ship ---*/
	ship->is_hit = TRUE;
	ship->hitcnt = 150;

	/*--- Update attacking ships nb ---*/
	if ( ship->is_attacking )
		currlevel.nb_ships_attacking--;

	}	// ship_is_hit
