// ============================================================================================
// ===																						===
// === test.c																				===
// ===																						===
// === Functions defined:																	===
// ===		test_btn_frames																	===
// ===		test_btn_bitmap																	===
// ===		test_btn_moveships																===
// ===		test_btn_moveshells																===
// ===		test_btn_rnd																	===
// ===																						===
// ============================================================================================

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <fcntl.h>
#include <sys/mman.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "phreshold.h"
#include "abimport.h"
#include "proto.h"


int
test_btn_frames( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	int			n;
	clock_t		t_start, t_end;
	UINT		t1, t2;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Draw 200 frames ---*/
	t_start = clock();
	for ( n = 0; n < 200; n++ )
		{
		PgSetRegion( PtWidgetRid( ABW_phreshold ) );
		PgDrawImagemx( frame, Pg_IMAGE_PALETTE_BYTE, &frame_pos, &frame_size, FRAME_WIDTH, 0 );
		PgFlush();
		}
	t_end = clock();

	/*--- How many times ---*/
	t1 = (t_end - t_start) / CLOCKS_PER_SEC;
	setString( ABW_test_infos_frames, "%d frames/sec", 200 / t1 );
	PtFlush();

	/*--- Draw 200 frames ---*/
	t_start = clock();
	for ( n = 0; n < 200; n++ )
		{
		memmove( frame, background, FRAME_SIZE );
		PgSetRegion( PtWidgetRid( ABW_phreshold ) );
		PgDrawImagemx( frame, Pg_IMAGE_PALETTE_BYTE, &frame_pos, &frame_size, FRAME_WIDTH, 0 );
		PgFlush();
		}
	t_end = clock();

	/*--- How many times ---*/
	t2 = (t_end - t_start) / CLOCKS_PER_SEC;
	setString( ABW_test_infos_frames, "%d frames/sec (%d with memmove)",
		200 / t1, 200 / t2 );

	return( Pt_CONTINUE );

	}	// test_btn_frames


int
test_btn_bitmap( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	time_t		start;
	int			nb;
	SHIP		*ship = &ships[ 0 ];

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Draw 10000 frames ---*/
	memmove( &currlevel, &conf_levels[ 0 ], sizeof( CONF_LEVEL ) );
	init_gen_ships();
	start = time( NULL );
	nb = 0;
	while ( time( NULL ) - start < 10 )
		{
		nb++;
		ship->x = rnd( 0, FRAME_WIDTH );
		ship->y = rnd( 0, FRAME_HEIGHT );
		blaster_or( currlevel.img[ 0 ], 
			currlevel.sz[ 0 ].w, currlevel.sz[ 0 ].h,
			ship->x, ship->y );
		}

	/*--- How many times ---*/
	setString( ABW_test_infos_bitmap, "%d bitmap/sec (%d micro-sec/bitmap)",
		nb/10, 10000000/nb );

	return( Pt_CONTINUE );

	}


int
test_btn_moveships( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	time_t		start;
	int			nb, n;
	SHELL		*shell;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Draw 10000 frames ---*/
	nb_active_shells = 0;
	for ( shell = shells, n = 0; n < MAXNB_SHELLS; n++, shell++ )
		shell->active = FALSE;
	memmove( &currlevel, &conf_levels[ 0 ], sizeof( CONF_LEVEL ) );
	init_gen_ships();
	start = time( NULL );
	nb = 0;
	while ( time( NULL ) - start < 10 )
		{
		nb++;
		move_ships();
		}

	/*--- How many times ---*/
	setString( ABW_test_infos_moveships, "%d move/sec (%d micro-sec/move)",
		nb/10, 1000000/nb );

	return( Pt_CONTINUE );

	}


int
test_btn_moveshells( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	int			fd;
	int			n, k;
	UBYTE		*p, *q;
	clock_t		t_start, t_end;
	UINT		t1, t2;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	if ( (fd = shm_open( "Physical", O_RDWR, 0777 )) == -1 )
		return( Pt_CONTINUE );

	vmem_frame = mmap( 0, 1280*1024, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0xE0000000 );
	if ( vmem_frame == (void*)-1 )
		return( Pt_CONTINUE );

	memset( frame, 0x12, 480*360 );

	t_start = clock();
	for ( n = 0; n < 1000; n++ )
		{
		p = frame;
		q = vmem_frame;
		for ( k = 0; k < 360; k++ )
			{
			memmove( q, p, 480 );
			p += 480;
			q += 1280;
			}
		}	// for (n)
	t_end = clock();

	/*--- How many times ---*/
	t1 = (t_end - t_start) / CLOCKS_PER_SEC;
	setString( ABW_test_infos_moveshells, "%d frames/sec", 1000 / t1 );
	PtFlush();

	t_start = clock();
	for ( n = 0; n < 1000; n++ )
		fastdraw_frame();
	t_end = clock();

	/*--- How many times ---*/
	t2 = (t_end - t_start) / CLOCKS_PER_SEC;
	setString( ABW_test_infos_moveshells, "%d frames/sec (%d using asm)",
		1000 / t1, 1000 / t2 );

	return( Pt_CONTINUE );

	}


int
test_btn_rnd( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	time_t		start;
	int			nb1, nb2, z;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- rand ---*/
	start = time( NULL );
	nb1 = 0;
	while ( time( NULL ) - start < 10 )
		{
		nb1++;
		z = (rand() % (500-100)) + 100;
		}
	setString( ABW_test_infos_rnd, "%d rand/s (%d micro-sec/rand)",
		nb1/10, 1000000/(nb1/10) );
	PtFlush();

	/*--- rand ---*/
	start = time( NULL );
	nb2 = 0;
	while ( time( NULL ) - start < 10 )
		{
		nb2++;
		z = rnd( 100, 500 );
		}
	setString( ABW_test_infos_rnd, 
		"%d rand/s (%d micro-sec/rand) - %d rnd/s (%d micro-sec/rnd)",
		nb1/10, 1000000/(nb1/10),
		nb2/10, 1000000/(nb2/10) );
	PtFlush();

	return( Pt_CONTINUE );

	}	// test_btn_rnd
