// ============================================================================================
// ===																						===
// === utils.c																				===
// ===																						===
// === Functions defined:																	===
// ===		message																			===
// ===		setString																		===
// ===		setPos																			===
// ===		setCursorType																	===
// ===		setGauge_MinMax																	===
// ===		setGauge_value																	===
// ===		getGauge_value																	===
// ===		setGauge_color																	===
// ===		getGauge_color																	===
// ===		window_focus																	===
// ===		blockWindow																		===
// ===		Bezier																			===
// ===		BSpline																			===
// ===																						===
// ============================================================================================

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "phreshold.h"
#include "abimport.h"
#include "proto.h"


/*--- -----------------------------------------------------------------	---*/
/*--- Function: message											 		---*/
/*--- 															 		---*/
/*--- Usage:	Print a message inside a dialog window. Just ok. 		---*/
/*--- -----------------------------------------------------------------	---*/
int message( const char *title, const char *fmt, ... )
	{
	va_list ap;
	char    buff[ 384 ];

	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );

	PtMessageBox( ABW_phreshold, title, buff, "helv14.phf", "  OK  " );

	return -1;

	}	// message


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void setString( const PtWidget_t *wgt, const char *fmt, ... )
	{
	PtArg_t		args[ 1 ];
	va_list		ap;
	char		buff[ 200 ];

	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );

	PtSetArg( &args[ 0 ], Pt_ARG_TEXT_STRING, buff, 0 );
	PtSetResources( wgt, 1, args );

	}	// setString


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void setPos( const PtWidget_t *wgt, const int x, const int y )
	{
	PtArg_t		args[ 1 ];
	PhPoint_t	pos;

	pos.x = x;
	pos.y = y;
	PtSetArg( &args[ 0 ], Pt_ARG_POS, &pos, 0 );
	PtSetResources( wgt, 1, args );

	}	// setPos


/*--- -----------------------------------------------------------------	---*/
/*--- Function: moveBy													---*/
/*--- 															 		---*/
/*--- Usage:	Change the dimension of the given widget.				---*/
/*--- -----------------------------------------------------------------	---*/
void moveBy( const PtWidget_t *wgt, const int dx, const int dy )
	{
	PtArg_t		args[ 1 ];
	PhPoint_t	*prevpos, newpos;

	PtSetArg( &args[ 0 ], Pt_ARG_POS, &prevpos, 0 );
	PtGetResources( wgt, 1, args );
	memmove( &newpos, prevpos, sizeof( PhPoint_t ) );
	newpos.x += dx;
	newpos.y += dy;
	PtSetArg( &args[ 0 ], Pt_ARG_POS, &newpos, 0 );
	PtSetResources( wgt, 1, args );

	}	// moveBy


/*--- -----------------------------------------------------------------	---*/
/*--- Function: setByDim												---*/
/*--- 															 		---*/
/*--- Usage:	Change the dimension of the given widget.				---*/
/*--- -----------------------------------------------------------------	---*/
void setByDim( const PtWidget_t *wgt, const int w, const int h )
	{
	PtArg_t		args[ 1 ];
	PhDim_t		*prevdim, newdim;

	PtSetArg( &args[ 0 ], Pt_ARG_DIM, &prevdim, 0 );
	PtGetResources( wgt, 1, args );
	memmove( &newdim, prevdim, sizeof( PhDim_t ) );
	newdim.w += w;
	newdim.h += h;
	PtSetArg( &args[ 0 ], Pt_ARG_DIM, &newdim, 0 );
	PtSetResources( wgt, 1, args );

	}	// setByDim


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void setCursorType( const PtWidget_t *wgt, int type )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_CURSOR_TYPE, type, 0 );
	PtSetResources( wgt, 1, args );

	}	// setCursorType


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void setGauge_MinMax( const PtWidget_t *wgt, 
					  const int vmin, const int vmax, const int vcurr )
	{
	PtArg_t	args[ 3 ];

	PtSetArg( &args[ 0 ], Pt_ARG_GAUGE_MINIMUM, vmin, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_GAUGE_MAXIMUM, vmax, 0 );
	PtSetArg( &args[ 2 ], Pt_ARG_GAUGE_VALUE,   vcurr, 0 );
	PtSetResources( wgt, 3, args );
	PtFlush();

	}	// setGauge_MinMax


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
int getGauge_value( const PtWidget_t *wgt )
	{
	PtArg_t	args[ 1 ];
	int		*value;

	PtSetArg( &args[ 0 ], Pt_ARG_GAUGE_VALUE, &value, 0 );
	PtGetResources( wgt, 1, args );
	return *value;

	}	// getGauge_value


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void setGauge_value( const PtWidget_t *wgt, const int value )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_GAUGE_VALUE, value, 0 );
	PtSetResources( wgt, 1, args );
	PtFlush();

	}	// setGauge_value


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void setGauge_color( const PtWidget_t *wgt, const PgColor_t color )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Rt_ARG_PROGRESS_BAR_COLOR, color, 0 );
	PtSetResources( wgt, 1, args );

	}	// setGauge_color


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
PgColor_t getGauge_color( const PtWidget_t *wgt )
	{
	PtArg_t		args[ 1 ];
	PgColor_t	*color;

	PtSetArg( &args[ 0 ], Rt_ARG_PROGRESS_BAR_COLOR, &color, 0 );
	PtGetResources( wgt, 1, args );

	return *color;
	}	// setGauge_color


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void window_focus( PtWidget_t *wgt )
	{
	PhWindowEvent_t		we;

	/*--- Send appropriate message to the window manager ---*/
	memset( &we, 0, sizeof( PhWindowEvent_t ) );
	we.rid         = PtWidgetRid( wgt );
	we.event_f     = Ph_WM_FOCUS;
	we.event_state = Ph_WM_EVSTATE_FOCUS;
	PtForwardWindowEvent( &we );
	
	}	// window_focus


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
int blockWindow( const PtWidget_t *wgt, const BOOL state )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_FLAGS, (state) ? Pt_TRUE : Pt_FALSE, Pt_BLOCKED );
	PtSetResources( wgt, 1, args );

	return Pt_CONTINUE;
	}	// blockWindow


/*----------------------------*/
/*--- Compute Bezier curve ---*/
/*----------------------------*/

static long C( const int n, const i )
	{
	long	a;
	int		j;
	int		ni = n - i;

	a = 1;
	for (j = i + 1; j <= n; j++)
		a *= j;

	for (j = 1; j <= ni; j++)
		a /= j;

	return (a);

	} // C


static double BBlend( const int i, const int n, const double u )
	{
	int			j;
	double		v;
    double		u1 = (1 - u);
	int			ni = n - i;

	v = C(n, i);

	for (j = 0; j < i; j++)
		v *= u;

	for (j = 0; j < ni; j++)
		v *= u1;;

	return (v); 

    } // BBlend


void Bezier( double *x, double *y, double u, int n, PhPoint_t *cc )
	{
	int		i;
	double	b;

	*x = 0;
	*y = 0;

	for (i = 0; i <= n; i++)
		{
		b = BBlend(i, n, u);
		*x += (cc[i].x * b);
		*y += (cc[i].y * b);
		} // for (i)

	} // Bezier


/*------------------------------*/
/*--- Compute B-Spline curve ---*/
/*------------------------------*/

static long Knot( const int i, int *knotK, int *knotN )
	{

	if (i < *knotK)
		return (0);
	else
		if (i > *knotN)
			return (*knotN - *knotK + 2);
		else
			return (i - *knotK + 1); 

    } // Knot


static double NBlend( const int i, const int k, const double u, int *knotK, int *knotN )
	{
	int		t;
	double	v;

	v = 0;
	if (k == 1)
		{
		if ( (Knot(i, knotK, knotN) <= u) && (u < Knot(i+1, knotK, knotN)) )
			v = 1; 
		} // if
	else
		{
		t = (int)(Knot(i+k-1, knotK, knotN) - Knot(i, knotK, knotN));
		if (t != 0)
			v = (u - Knot(i, knotK, knotN)) * NBlend(i, k-1, u, knotK, knotN) / t;
		t = (int)(Knot(i+k, knotK, knotN) - Knot(i+1, knotK, knotN));
		if (t != 0)
			v += ((Knot(i+k, knotK, knotN) - u) * NBlend(i+1, k-1, u, knotK, knotN) / t);
        } // else

    return (v);

    } // NBlend


void BSpline( double *x, double *y, double u, int n, int k, PhPoint_t *cc )
	{
	int		knotK, knotN;
	int		i;
    double	b;

	*x = 0;
	*y = 0;

	knotK = k;
    knotN = n;

	for (i = 0; i <= n; i++)
		{
		b = NBlend(i, k, u, &knotK, &knotN);
		*x += (cc[i].x * b);
		*y += (cc[i].y * b);
		} // for (i)

	} // BSpline


void RectUnion( PhRect_t *r, const PhRect_t *r1, const PhRect_t *r2 )
	{
	r->ul.x = min( r1->ul.x, r2->ul.x );	
	r->ul.y = min( r1->ul.y, r2->ul.y );	
	r->lr.x = max( r1->lr.x, r2->lr.x );
	r->lr.y = max( r1->lr.y, r2->lr.y );
	}	// RectUnion


static int query_input_group(int input_group, PhRect_t *rect)
	{
	int			nrids, cnt;
	PhRid_t		*rids;
	PhRegion_t	rInfo;

	nrids = PhQueryRids( 0, 0, input_group, Ph_INPUTGROUP_REGION, 0, 0, 0, 0, 0 );

	if( nrids == 0 || nrids == -1 )
		return -1;
	
	/* allocate memory for rids */
	if( (rids = ( PhRid_t *)malloc( sizeof( PhRid_t ) * nrids ) ) == NULL ) 
		{
		perror( "PWM: cannot allocate memory for rids\n" );
		return -1;
		}

	nrids = PhQueryRids( 0, 0, input_group, Ph_INPUTGROUP_REGION, 0, 0, 0, rids, nrids );

	for( cnt = 0; cnt < nrids; cnt++ ) 
		{
		if( PhRegionQuery( rids[cnt], &rInfo, rect, 0, 0 ) == -1 ) 
			continue;
		if( rInfo.flags & Ph_INPUTGROUP_REGION )
			break;
		}
	free( rids );
	return 0;
	}	// query_input_group


int force_coord_into_console( int x, int y, int *px, int *py )
	{
	PhRect_t	r;

	if ( query_input_group( 1, &r ) == -1 )
		{
		*px = x;
		*py = y;
		return -1;
		}

	*px = (x % r.lr.x);
	*py = (y % r.lr.y);
	return 0;

	}	// force_coord_into_console

